""" Early initialization. """

from os.path import exists
from shutil import copyfile
from base64 import b64decode
from os import getcwd, system
from redis import StrictRedis
from yaml import load, FullLoader
from asyncio import get_event_loop
from coloredlogs import ColoredFormatter
from discord import DMChannel
from discord.ext.commands import AutoShardedBot, HelpCommand, when_mentioned_or
from logging import getLogger, StreamHandler, INFO, DEBUG

class Bot(AutoShardedBot):
    def __init__(self, *args, prefix=None, **kwargs):
        super().__init__(*args, **kwargs)
        self.prefix = prefix

    async def on_message(self, message):
        if not self.is_ready() or message.author.bot or not isinstance(message.channel, DMChannel) or \
           message.channel.permissions_for(message.guild.me).send_messages or \
           message.author.id in system_administrators or message.author.id in operators:
            return
        await self.process_commands(message)


first_run = False
if not exists("config.yml"):
    copyfile(f"{__path__[0]}/assets/config-sample.yml", "config.yml")
    first_run = True


# noinspection PyUnusedLocal
async def get_prefix(bot, context):
    if not redis_status():
        return when_mentioned_or(*[default_prefix])(bot, context)
    guild = context.guild
    if guild:
        prefix = redis.hget(f"settings.{guild.id}", f"prefix")
        if prefix:
            return when_mentioned_or(*[b64decode(prefix.decode("utf-8")).decode("utf-8")])(bot, context)
        else:
            return when_mentioned_or(*[default_prefix])(bot, context)
    else:
        return when_mentioned_or(*[default_prefix])(bot, context)

loop = get_event_loop()
module_dir = __path__[0]
working_dir = getcwd()
config = load(open("config.yml"), Loader=FullLoader)
token = config.get("token", None)
system_administrators = config.get("system_administrators", None)
operators = config.get("operators", None)
default_prefix = config.get("default_prefix", "env ")
debug = config.get("debug", False)
presence = config.get("presence", None)
logger = getLogger(__name__)
handler = StreamHandler()
handler.setFormatter(ColoredFormatter("[%(asctime)s] %(levelname)s %(message)s"))
logger.addHandler(handler)
logger.setLevel(INFO) if not config.get("verbose", False) else logger.setLevel(DEBUG)
redis_connection_info = config.get("redis", None)
leaderboards = {}
invite_url = []
leaderboard_status = {}
mod_apply_status = {}
nitori = AutoShardedBot(
    command_prefix=get_prefix,
    case_insensitive=False,
    pm_help=False,
    help_attrs=dict(hidden=True),
    formatter=None
)
redis = StrictRedis(host=redis_connection_info.get("host", "localhost"),
                    port=redis_connection_info.get("port", 6379),
                    db=redis_connection_info.get("db", 7)
                    ) if redis_connection_info is not None and isinstance(redis_connection_info, dict) else None
system("rm -rf /tmp/nitori_cache")
system("mkdir /tmp/nitori_cache")

if first_run:
    logger.fatal("Generated configuration file, please edit it before starting aya.")
    exit(1)

if redis is None or token is None or token == "NzUxODM5OTAxNjE5NTg1MTg1.X1O7eQ.SLfND-Rye51JJwPbs_h-CLDDCUo":
    logger.fatal("Please edit the configuration file before starting aya.")
    exit(1)


class HelpFormat(HelpCommand):
    async def format_help_for(self, context, command_or_bot):
        return await super().prepare_help_command(context, command_or_bot)


def redis_status():
    try:
        redis.ping()
        return True
    except BaseException:
        return False
