""" Nitori module bootloader. """

from sys import path
from os import remove
from os.path import exists
from discord import LoginFailure
from asyncio import get_event_loop
from importlib import import_module
from concurrent.futures import ThreadPoolExecutor
from aya import logger, token, nitori, working_dir
from aya.internal import internal_list, plugin_list
from aya.utils import cleanup

path.insert(1, f"{working_dir}/plugins")
load_successful = True
logger.info("Loading modules...")
_executor = ThreadPoolExecutor(10)
loop = get_event_loop()

for module in internal_list:
    load_successful = True
    try:
        nitori.load_extension(f"aya.internal.{module}")
    except BaseException as exception:
        logger.info(f"Error loading {module}: {exception}")
        internal_list.remove(module)
        load_successful = False
        pass

for module in plugin_list:
    try:
        nitori.load_extension(f"plugins.{module}")
    except BaseException as exception:
        logger.info(f"Error loading {module}: {exception}")
        plugin_list.remove(module)
        load_successful = False
        pass

logger.info("Module loading finished.")

if not load_successful:
    logger.warning("Some modules loaded with errors, please check previous log messages.")

# TODO: Implement a real reload feature.

while not exists("/tmp/nitori_shutdown"):
    import_module("aya.frontend")
    logger.info("Connecting to discord...")
    try:
        nitori.run(token)
    except LoginFailure:
        logger.fatal("Invalid bot token specified in configuration.")
        exit(1)
    except BaseException as exception:
        logger.fatal(f"Exception: {exception}")
        exit(1)
    if not exists("/tmp/nitori_shutdown"):
        logger.info("Forcefully shutting down...")
        cleanup.frontend_cleanup()
        break


if exists("/tmp/nitori_shutdown"):
    remove("/tmp/nitori_shutdown")
    logger.info("Nitori has successfully shut down.")
