""" Aya frontend. """

from time import sleep
from threading import Thread
from importlib import import_module
from werkzeug.middleware.proxy_fix import ProxyFix
from flask import Flask
from flask.logging import default_handler
from flask_bcrypt import Bcrypt
from flask_login import LoginManager
from flask_discord import DiscordOAuth2Session
from cheroot.wsgi import Server as WSGIServer, PathInfoDispatcher
from aya import config, logger, handler, nitori


app = Flask("aya")
app.config["CSRF_ENABLED"] = True
app.config["TEMPLATES_AUTO_RELOAD"] = True
app.config["SECRET_KEY"] = config["web_frontend"]["secret_key"]
app.config["DISCORD_CLIENT_ID"] = config['client_id']
app.config["DISCORD_CLIENT_SECRET"] = config['client_secret']
app.config["DISCORD_REDIRECT_URI"] = f"{config['url']}callback"
app.wsgi_app = ProxyFix(app.wsgi_app)
bcrypt = Bcrypt(app)
login = LoginManager()
login.init_app(app)
discord = DiscordOAuth2Session(app)


@app.before_first_request
def wait_for_session():
    while not nitori.is_ready():
        sleep(.5)


import_module('aya.frontend.routes')

dispatcher = PathInfoDispatcher({'/': app})
server = WSGIServer((config['web_frontend']['host'], int(config['web_frontend']['port'])), dispatcher)


def start():
    if config['web_frontend'].get("enabled", False):
        logger.info(f"Web server listening at {config['web_frontend']['host']}:{config['web_frontend']['port']}")
        app.logger.removeHandler(default_handler)
        app.logger.addHandler(handler)
        try:
            server.start()
        except OSError:
            logger.fatal(f"Another process is listening on port {config['web_frontend']['port']}!")
            return
    else:
        logger.warning("Web frontend is disabled explicitly via configuration.")


Thread(target=start).start()
