""" Aya web interface input forms. """

from flask_wtf import FlaskForm
from wtforms import StringField, BooleanField, ValidationError
from wtforms.validators import Length, Optional, URL


def channel_id(form, field):
    if len(field.data) != 18:
        raise ValidationError("Field must have the length of exactly 18.")
    elif not field.data.isnumeric():
        raise ValidationError("Field has to be an integer.")


class MessageModifyForm(FlaskForm):
    prefix = StringField("Prefix", validators=[Length(min=0, max=16, message="Input exceeds limit, "
                                                                             "please specify a string within "
                                                                             "16 characters."), Optional()])
    levelup = StringField("Level Up", validators=[Length(min=0, max=1024, message="Input exceeds limit, "
                                                                                  "please specify a string within "
                                                                                  "1024 characters."), Optional()])
    notification_channel = StringField("Notification Channel ID", validators=[channel_id, Optional()])
    welcome_message = StringField(
        "Welcome Message", validators=[Length(min=0, max=1024, message="Input exceeds limit, "
                                                                       "please specify a string within "
                                                                       "1024 characters."), Optional()])
    welcome_image_url = StringField(
        "Welcome Image URL", validators=[Length(min=0, max=1024, message="Input exceeds limit, "
                                                                         "please specify a string within "
                                                                         "1024 characters."),
                                         URL(require_tld=True, message="Please enter a valid URL"), Optional()])
    goodbye_message = StringField(
        "Welcome Message", validators=[Length(min=0, max=1024, message="Input exceeds limit, "
                                                                       "please specify a string within "
                                                                       "1024 characters."), Optional()])
    goodbye_image_url = StringField(
        "Goodbye Image URL", validators=[Length(min=0, max=1024, message="Input exceeds limit, "
                                                                         "please specify a string within "
                                                                         "1024 characters."),
                                         URL(require_tld=True, message="Please enter a valid URL"), Optional()])


class LevelingSystemForm(FlaskForm):
    enabled = BooleanField("Enable")
