""" All logging-related stuff is done here. """

from discord.ext.commands import Cog, guild_only, command
from aya import redis_status
from aya.utils import permissions, customize, formatter


class Logging(Cog):
    """ log_desc """

    @command(aliases=["logging", "logger"])
    @guild_only()
    @permissions.has_permissions(administrator=True)
    async def log(self, context):
        """ log_help_log """
        if customize.get_log_channel(context.guild.id) == context.channel:
            customize.remove_log_channel(context.guild.id)
            await context.send("Log channel has been unset.")
        else:
            customize.set_log_channel(context.guild.id, context.channel)
            await context.send("Log channel has been set to current channel.")

    @Cog.listener()
    async def on_message_edit(self, pre, post):
        if not hasattr(pre, "guild"):
            return
        elif pre.guild is None:
            return
        await formatter.generate_log_message(pre, formatter.build_message_log("Message Edit", pre, post))

    @Cog.listener()
    async def on_message_delete(self, pre):
        if not hasattr(pre, "guild"):
            return
        elif pre.guild is None:
            return
        await formatter.generate_log_message(pre, formatter.build_message_log("Message Delete", pre))

    @Cog.listener()
    async def on_member_update(self, pre, post):
        if not hasattr(pre, "guild"):
            return
        elif pre.guild is None:
            return
        if pre.nick != post.nick:
            await formatter.generate_log_message(pre,
                                                 formatter.build_string_log("Nickname Change", pre, pre.nick,
                                                                            post.nick))


def setup(client):
    if redis_status():
        client.add_cog(Logging())
