""" Nitori manuals. """

from discord import Embed, Color
from discord.ext.commands import Cog, command
from aya import nitori, default_prefix
from aya.utils import customize, localization


class Manuals(Cog):
    """ man_h_desc """

    @command(aliases=["help"])
    async def man(self, context, *modules):
        """ man_h_help_man """
        try:
            prefix = customize.get_prefix(context.guild.id)
        except AttributeError:
            prefix = default_prefix
        if not prefix:
            prefix = default_prefix
        if not modules:
            embed = Embed(title=localization.get_string("man_title", context),
                          description=localization.get_string("man_desc", context).format(prefix=prefix),
                          color=Color.blue())
            module_description = ""
            for cog in nitori.cogs:
                if str(cog) in ["System", "Handler", "Mask"]:
                    continue
                try:
                    description = localization.get_string(nitori.cogs[cog].__doc__.replace(' ', ''), context)
                except RuntimeError:
                    description = nitori.cogs[cog].__doc__
                module_description += f"{str(cog)} - " \
                                      f"{description}\n"
            embed.add_field(name=localization.get_string("man_modules", context),
                            value=module_description[0:len(module_description) - 1], inline=False)

            await context.send(embed=embed)
        else:
            if len(modules) > 1:
                return await context.send(localization.get_string("man_error_multi", context))
            else:
                for cog in nitori.cogs:
                    for module in modules:
                        if cog.lower() == module.lower():
                            try:
                                description = localization.get_string(nitori.cogs[cog].__doc__.replace(" ", ""),
                                                                      context)
                            except RuntimeError:
                                description = nitori.cogs[cog].__doc__
                            embed = Embed(
                                title=localization.get_string("man_module_manual_title", context).format(module=cog),
                                description=description)
                            for command_entry in nitori.get_cog(cog).get_commands():
                                if not command_entry.hidden:
                                    try:
                                        embed.add_field(name=command_entry.name,
                                                        value=localization.get_string(
                                                            command_entry.help.replace(" ", ""),
                                                            context),
                                                        inline=False)
                                    except AttributeError:
                                        embed.add_field(name=command_entry.name,
                                                        value=localization.get_string("command_help_unavail", context),
                                                        inline=False)
                            return await context.send(embed=embed)
                return await context.send(localization.get_string("man_error_not_found", context))


def setup(client):
    client.remove_command('help')
    client.add_cog(Manuals())
