""" Experience based ranking system. """

from random import randrange
from discord import Member, Embed
from discord.utils import get
from discord.errors import Forbidden
from discord.ext.commands import Cog, command, guild_only
from aya import nitori, config, logger, redis_status, get_prefix
from aya.utils import experience, customize, permissions, localization


class Rank(Cog):
    """ exp_desc """

    @Cog.listener()
    async def on_message(self, context):
        if context.author.bot:
            return
        try:
            gid = context.guild.id
            uid = context.author.id
            channel = context.channel.id
        except AttributeError:
            return
        if customize.get_exp_bl(gid, channel):
            return
        prefix_array = await get_prefix(nitori, context)
        for prefix in prefix_array:
            if context.content.startswith(prefix):
                return
        if not experience.get_exp(uid, gid):
            experience.set_exp(uid, gid, 0)
        current_exp = experience.get_exp(uid, gid)
        current_level = experience.calculate_level(current_exp)
        new_exp = current_exp + randrange(5, 10, 1)
        new_level = experience.calculate_level(new_exp)
        experience.set_exp(uid, gid, new_exp)
        message = customize.get_message(gid, "levelup")
        if new_level == current_level + 1:
            try:
                formatted_message = message.format(user=f"<@{uid}>", level=str(
                    new_level)) if message else config.get("default_levelup_message",
                                                           "Congratulations {user} on reaching level {level}.").format(
                    user=f"<@{uid}>", level=str(
                        new_level))
            except KeyError:
                formatted_message = f"Congratulations <@{uid}> on reaching level {new_level}."
            try:
                await context.channel.send(formatted_message)
            except Forbidden:
                pass
            if customize.get_rank(gid, new_level):
                try:
                    role = get(context.guild.roles, id=int(customize.get_rank(gid, new_level)))
                    await context.author.add_roles(role)
                except BaseException:
                    await context.channel.send(
                        localization.get_string("exp_role_missing", context).format(
                            role=str(customize.get_rank(gid, new_level)),
                            level=new_level)
                    )
                    customize.remove_rank(gid, new_level)

    @command(aliases=['exp', 'xp', 'rank'])
    @guild_only()
    async def level(self, context, *, user: Member = None):
        """ exp_help_rank """
        if user is None:
            user = context.author
        if user.bot:
            await context.send(localization.get_string("invalid_argument", context))
            return
        try:
            gid = context.guild.id
            uid = user.id
        except AttributeError:
            return
        rank = 1
        exp = experience.get_exp(uid, gid)
        if not exp:
            exp = 0
        for member in context.guild.members:
            if member.bot or member == user:
                continue
            if experience.get_exp(member.id, gid) >= exp:
                rank += 1
        level = experience.calculate_level(exp)
        next_level = experience.calculate_exp(level + 1)
        current_level_exp = experience.calculate_exp(level)
        content = Embed(title=localization.get_string("exp_level_title", context),
                        description=str(user),
                        colour=user.color)
        content.add_field(name=localization.get_string("exp_level_rank", context), value="#" + str(rank), inline=True)
        content.add_field(name=localization.get_string("exp_level_level", context), value=level, inline=True)
        content.add_field(name=localization.get_string("exp_level_exp", context),
                          value=f"{exp - current_level_exp}/{next_level - current_level_exp}",
                          inline=True)
        content.set_thumbnail(url=user.avatar_url_as(format=None, static_format="png", size=128))
        await context.send(embed=content)

    @command()
    @guild_only()
    @permissions.system_administrators_only()
    async def rank_sync(self, context):
        """ exp_help_rank_sync """
        message = await context.send(localization.get_string("exp_sync_roles", context))
        assigned_users = 0
        members = context.guild.members
        for user in members:
            if user.bot:
                continue
            uid = user.id
            gid = context.guild.id
            exp = experience.get_exp(uid, gid)
            if not exp:
                experience.set_exp(uid, gid, 0)
                exp = 0
            level = experience.calculate_level(exp)
            assigned = False
            ranks = customize.get_ranks(gid)
            for rank in ranks:
                try:
                    ranked_role = get(context.guild.roles, id=ranks[rank])
                except AttributeError:
                    customize.remove_rank(gid, int(rank))
                    continue
                await user.remove_roles(ranked_role)
                if level >= int(rank):
                    await user.add_roles(ranked_role)
                    assigned = True
            assigned_users += 1 if assigned else 0
        await message.delete()
        await context.send(localization.get_string("exp_sync_done", context).format(assigned_users=assigned_users,
                                                                                    total_users=len(members)))

    @command()
    @guild_only()
    async def leaderboard(self, context):
        """ exp_help_leaderboard """
        content = Embed(title=localization.get_string("exp_leaderboard_title", context),
                        description=localization.get_string("exp_leaderboard_body", context).format(
                            link=f"[{localization.get_string('exp_leaderboard_link', context)}]("
                                 f"{config.get('url', 'http://localhost:7777/')}leaderboard/{context.guild.id})"))
        await context.send(embed=content)


def setup(client):
    if config.get("rank", True) and redis_status():
        client.add_cog(Rank())
    else:
        logger.warning("The ranking system is disabled due to database error or explicitly set in configuration.")
