""" Shutdown cleanup related stuff. """

from asyncio import sleep
from pathlib import Path
from aya import leaderboard_status, nitori, logger
from aya.frontend import server
from aya.internal import internal_list, internal_string
from aya.utils import voice


async def clean_shutdown():
    frontend_cleanup()
    await voice_pool_cleanup()
    Path("/tmp/nitori_shutdown").touch()
    await nitori.close()


async def voice_pool_cleanup():
    logger.info("Disconnecting from voice channels...")
    for gid in voice.session_pool:
        session = voice.get_session(gid)
        await session.cleanup()


async def reload_internals(context):
    await sleep(.01)
    logger.info(f"Reloading internal modules: {internal_string}")
    load_successful = True
    for module in internal_list:
        try:
            nitori.unload_extension(f"nitori.internal.{module}")
            nitori.load_extension(f"nitori.internal.{module}")
        except BaseException as exception:
            logger.info(f"Error loading {module}: {exception}")
            internal_list.remove(module)
            load_successful = False
            pass
        if not load_successful:
            return await context.edit("Reloaded with errors, check application logs for more information.")
        return await context.edit(content="Successfully performed reload of all internal modules.")


def frontend_cleanup():
    logger.info("Shutting down web server and leaderboard builder...")
    if server is not None:
        server.stop()
    rebuild_thread = leaderboard_status.get("rebuild_thread", None)
    if rebuild_thread is not None:
        rebuild_thread.cancel()
