""" Leaderboard building utility. """

from time import time
from threading import Timer
from aya import nitori, leaderboards, logger, leaderboard_status, config, loop
from aya.utils import experience


def periodical_reload():
    rebuild_thread = Timer(int(config.get("leaderboard_refresh", "3600")), periodical_reload)
    leaderboard_status.update({"rebuild_thread": rebuild_thread})
    rebuild_thread.start()
    try:
        loop.create_task(rebuild_leaderboards())
    except RuntimeError:
        pass


async def rebuild_leaderboards():
    startup = leaderboard_status.get("startup", False)
    if leaderboard_status.get("another_startup_is_running", False):
        return
    if startup:
        leaderboard_status.update({"another_startup_is_running": True})
    logger.debug(f"Started routine leaderboard rebuild for {str(len(nitori.guilds))} guilds.")
    start_time = time()
    for guild in nitori.guilds:
        gid = guild.id
        if leaderboard_status.get(gid, False) or startup:
            experiences = experience.get_exps(gid)
            levels = sorted(experiences, key=experiences.get, reverse=True)
            if gid not in leaderboards:
                leaderboards.update({gid: {"users": [], "updated": 0}})
            await leaderboard_builder(levels, gid)
        else:
            logger.warning(
                f"Another rebuild is already running for guild {gid}, maybe the server is overloaded or the interval "
                f"is too short? Skipping this rebuild.")
    logger.debug(f"Rebuild finished elapsing {time() - start_time} seconds.")
    if startup:
        del(leaderboard_status["startup"])
        del(leaderboard_status["another_startup_is_running"])


async def leaderboard_builder(levels, gid):
    logger.debug(f"Started rebuild of leaderboard for guild {str(gid)}.")
    leaderboard_status.update({gid: False})
    leaderboards[gid]["users"] = []
    users = leaderboards[gid]["users"]
    rank = 0
    guild = nitori.get_guild(int(gid))
    user_list = {}
    for user in guild.members:
        if user.bot:
            continue
        user_list.update({user.id: user})
    for uid in levels:
        rank += 1
        user = user_list.get(int(uid), None)
        if user is None:
            continue
        exp = experience.get_exp(int(uid), gid)
        user_dictionary = dict(uid=user.id,
                               username=user.name,
                               display_name=user.display_name,
                               profile_picture=user.avatar_url_as(format=None, static_format="png", size=128),
                               exp=exp,
                               rank=rank,
                               level=experience.calculate_level(exp),
                               discriminator=user.discriminator)
        users.append(user_dictionary)
        logger.debug(f"Leaderboard entry updated for guild {str(gid)}.")
    logger.debug(f"Finished leaderboard rebuild for guild {str(gid)}.")
    leaderboard_status.update({gid: True})
