""" Permission checker for context. """


from discord import DMChannel
from discord.ext import commands
from discord.ext.commands.errors import CheckFailure
from aya.utils import operators, system_administrators, localization


async def check_permission(context, perms, *, check=all):
    if context.author.id in system_administrators or context.author.id in operators:
        return True
    return check(getattr(context.channel.permissions_for(context.author), name, None)
                 == value for name, value in perms.items())


def verify_system_administrator(context):
    return context.author.id in system_administrators


def verify_operator(context):
    return context.author.id in system_administrators or context.author.id in operators


def has_permissions(*, check=all, **perms):
    async def check_handler(context):
        if not await check_permission(context, perms, check=check):
            raise CheckFailure(localization.get_string("permission_denied", context))
        return True
    return commands.check(check_handler)


def operator_only():
    async def check_handler(context):
        if not verify_operator(context):
            raise CheckFailure(localization.get_string("operator_only", context))
        return True
    return commands.check(check_handler)


def system_administrators_only():
    async def check_handler(context):
        if not verify_system_administrator(context):
            raise CheckFailure(localization.get_string("admin_only", context))
        return True
    return commands.check(check_handler)


def send(context):
    return isinstance(context.channel, DMChannel) or \
           context.channel.permissions_for(context.guild.me).send_messages or \
           context.author.id in system_administrators or context.author.id in operators


def embed(context):
    return isinstance(context.channel, DMChannel) or \
           context.channel.permissions_for(context.guild.me).embed_links or \
           context.author.id in system_administrators or context.author.id in operators


def attach(context):
    return isinstance(context.channel, DMChannel) or \
           context.channel.permissions_for(context.guild.me).attach_files or \
           context.author.id in system_administrators or context.author.id in operators


def react(context):
    return isinstance(context.channel, DMChannel) or \
           context.channel.permissions_for(context.guild.me).add_reactions or \
           context.author.id in system_administrators or context.author.id in operators


def invite(context):
    return isinstance(context.channel, DMChannel) or \
           context.channel.permissions_for(context.guild.me).create_invite or \
           context.author.id in system_administrators or context.author.id in operators


def manage(context):
    return not isinstance(context.channel, DMChannel) and \
           context.channel.permissions_for(context.guild.me).manage_messages or \
           context.author.id in system_administrators or context.author.id in operators


def roles(context):
    return not isinstance(context.channel, DMChannel) and \
           context.channel.permissions_for(context.guild.me).manage_roles or \
           context.author.id in system_administrators or context.author.id in operators


def nsfw(context):
    return isinstance(context.channel, DMChannel) or context.channel.is_nsfw()
