""" Touhou utils. """

from asyncio import sleep
from discord import Embed
from random import randint
from bs4 import BeautifulSoup
from aiohttp import ClientSession


all_characters = {"Reimu": ["Hakurei_Reimu", 0xb50404],
                  "Marisa": ["Kirisame_Marisa", 0xf5e942],
                  "Tenshi": ["Hinanawi_Tenshi", 0x42d4f4],
                  "Sakuya": ["Izayoi_Sakuya", 0xc7c7c7],
                  "Cirno": ["Cirno", 0x00e5ff], "Vert": ["Cirno", 0x00e5ff],
                  "Meiling": ["Hong_Meiling", 0x04b548],
                  "Flandre": ["Flandre_Scarlet", 0xb50404],
                  "Rumia": ["Rumia", 0xf5da42],
                  "Rinnosuke": ["Morichika_Rinnosuke", 0x42D4F4],
                  "Murasa": ["Murasa_Minamitsu", 0x42D4F4],
                  "Mamizou": ["Futatsuiwa_Mamizou", 0x42D4F4],
                  "Shou": ["Toramaru_Shou", 0x42D4F4],
                  "Nemuno": ["Sakata_Nemuno", 0x42D4F4],
                  "Eternity": ["Eternity_Larva", 0x42D4F4],
                  "Narumi": ["Yatadera_Narumi", 0x42D4F4],
                  "Daiyousei": ["Daiyousei", 0x04b548],
                  "Ringo": ["Ringo_Touhou", 0xe2a81e],
                  "Kosuzu": ["Motoori_Kosuzu", 0x42D4F4],
                  "Akyuu": ["Hieda_No_Akyuu", 0x42D4F4],
                  "Hatate": ["Himekaidou_Hatate", 0xb50480],
                  "Mima": ["Mima", 0x42D4F4],
                  "Sariel": ["Sariel", 0x42D4F4],
                  "Yumemi": ["Okazaki_Yumemi", 0x42D4F4],
                  "Shinki": ["Shinki", 0x42D4F4],
                  "Lily": ["Lily_White", 0x42D4F4],
                  "Shion": ["Yorigami_Shion", 0x048cb5],
                  "Joon": ["Yorigami_Joon", 0xaa4fa0],
                  "Seiran": ["Seiran_(touhou)", 0x6b87bd],
                  "Koakuma": ["Koakuma", 0x990000],
                  "Raiko": ["Horikawa_Raiko", 0xd25859],
                  "Okina": ["Matara_Okina", 0xe69454],
                  "Mai": ["Teireida_Mai", 0x4e7764],
                  "Satono": ["Nishida_Satono", 0xe262b0],
                  "Aunn": ["Komano_Aun", 0x42D4F4],
                  "Komachi": ["Onozuka_Komachi", 0xd25859],
                  "Wakasagihime": ["Wakasagihime", 0x42D4F4],
                  "Seija": ["Kijin_Seija", 0xaeb4c6],
                  "Toyohime": ["Watatsuki_No_Toyohime", 0x583b80],
                  "Yorihime": ["Watatsuki_No_Yorihime", 0xa84384],
                  "Renko": ["Usami_Renko", 0x42D4F4],
                  "Maribel": ["Maribel_Hearn", 0x42D4F4],
                  "Nue": ["Houjuu_Nue", 0x000000],
                  "Iku": ["Nagae_Iku", 0x42D4F4],
                  "Elly": ["Elly", 0x42D4F4],
                  "Kasen": ["Ibaraki_Kasen", 0xfb959e],
                  "Keine": ["Kamishirasawa_Keine", 0x574b8c],
                  "Konngara": ["Konngara", 0x42D4F4],
                  "Yuyuko": ["Saigyouji_Yuyuko", 0xff40d9],
                  "Aya": ["Shameimaru_Aya", 0xe58a53],
                  "Nitori": ["Kawashiro_Nitori", 0xb2daef],
                  "Sumireko": ["Usami_Sumireko", 0xaa6ad3],
                  "Okuu": ["Reiuji_Utsuho", 0x009917],
                  "Patchouli": ["Patchouli_Knowledge", 0xc646e0],
                  "Youmu": ["Konpaku_Youmu", 0x79eb50],
                  "Koishi": ["Komeiji_Koishi", 0x62f500],
                  "Mokou": ["Fujiwara_No_Mokou", 0xf50000],
                  "Satori": ["Komeiji_Satori", 0xa700f5],
                  "Wan": ["Inubashiri_Momiji", 0x42D4F4],
                  "Momiji": ["Inubashiri_Momiji", 0x42D4F4],
                  "Ran": ["Yakumo_Ran", 0x42D4F4],
                  "Kagerou": ["Imaizumi_Kagerou", 0x42D4F4],
                  "Reisen": ["Reisen_Udongein_Inaba", 0xf94aff],
                  "Rei": ["Reisen", 0x2291ba],
                  "Letty": ["Letty_Whiterock", 0x42D4F4],
                  "Remilia": ["Remilia_Scarlet", 0xfd8cff],
                  "Suwako": ["Moriya_Suwako", 0x42D4F4],
                  "Shizuha": ["Aki_Shizuha", 0x42D4F4],
                  "Sanae": ["Kochiya_Sanae", 0x24b343],
                  "Clownpiece": ["Clownpiece", 0x42D4F4],
                  "Yukari": ["Yakumo_Yukari", 0x42D4F4],
                  "Yuuka": ["Kazami_Yuuka", 0x24b343],
                  "Suika": ["Ibuki_Suika", 0x42D4F4],
                  "Sekibanki": ["Sekibanki", 0x42D4F4],
                  "Wriggle": ["Wriggle_Nightbug", 0x42D4F4],
                  "Hina": ["Kagiyama_Hina", 0x42D4F4],
                  "Alice": ["Alice_Margatroid", 0x42D4F4],
                  "Kyouko": ["Kasodani_Kyouko", 0x42D4F4],
                  "Kisume": ["Kisume", 0x42D4F4],
                  "Nazrin": ["Nazrin", 0x42D4F4],
                  "Sukuna": ["Sukuna_Shinmyoumaru", 0x42D4F4],
                  "Kokoro": ["Hata_No_Kokoro", 0x42D4F4],
                  "Yoshika": ["Miyako_Yoshika", 0x42D4F4],
                  "Seiga": ["Kaku_Seiga", 0x42D4F4],
                  "Kogasa": ["Tatara_Kogasa", 0x42D4F4],
                  "Futo": ["Mononobe_No_Futo", 0x42D4F4],
                  "Miko": ["Toyosatomimi_No_Miko", 0x42D4F4],
                  "Mystia": ["Mystia_Lorelei", 0xff8ade],
                  "Genjii": ["Genjii", 0x42D4F4],
                  "Byakuren": ["Hijiri_Byakuren", 0x5b0082],
                  "Hecatia": ["Hecatia_Lapislazuli", 0x940f0f],
                  "Junko": ["Junko_(touhou)", 0xfbd55a],
                  "Sagume": ["Kishin_Sagume", 0xb4449c],
                  "Doremy": ["Doremy_Sweet", 0x42D4F4],
                  "Minoriko": ["Aki_Minoriko", 0x42D4F4],
                  "Yamame": ["Kurodani_Yamame", 0x42D4F4],
                  "Yuugi": ["Hoshiguma_Yuugi", 0x42D4F4],
                  "Parsee": ["Mizuhashi_Parsee", 0x42D4F4],
                  "Tewi": ["Inaba_Tewi", 0xcc7c9c],
                  "Medicine": ["Medicine_Melancholy", 0x42D4F4],
                  "Eiki": ["Shiki_Eiki", 0x5b9c66],
                  "Orin": ["Kaenbyou_Rin", 0x42D4F4],
                  "Kaguya": ["Houraisan_Kaguya", 0xef61ff],
                  "Eirin": ["Yagokoro_Eirin", 0x42D4F4],
                  "Kanako": ["Yasaka_Kanako", 0x977cac],
                  "Chen": ["Chen", 0x42D4F4],
                  "Star": ["Star_Sapphire", 0x42D4F4],
                  "Luna": ["Luna_Child", 0x42D4F4],
                  "Sunny": ["Sunny_Milk", 0x42D4F4],
                  "Eika": ["Ebisu_Eika", 0x42D4F4],
                  "Urumi": ["Ushizaki_Urumi", 0x42D4F4],
                  "Kutaka": ["Niwatari_Kutaka", 0xdf9041],
                  "Lunasa": ["Lunasa_Prismriver", 0x42D4F4],
                  "Lyrica": ["Lyrica_Prismriver", 0x42D4F4],
                  "Merlin": ["Merlin_Prismriver", 0x42D4F4],
                  "Keiki": ["Haniyasushin_Keiki", 0x42D4F4],
                  "Saki": ["Kurokoma_Saki", 0x42D4F4],
                  "Mayumi": ["Joutouguu_Mayumi", 0x42D4F4],
                  "Yachie": ["Kicchou_Yachie", 0x42D4F4],
                  "Ichirin": ["Kumoi_Ichirin", 0x42D4F4],
                  "Miyoi": ["Okunoda_Miyoi", 0x42D4F4],
                  "Chiyuri": ["Kitashirakawa_Chiyuri", 0x42D4F4],
                  "Youka": ["Kazami_Youka", 0x42D4F4],
                  "Kokuu": ["Kokuu_Haruto", 0x14a625],
                  "Ex_Rumia": ["Ex-Rumia", 0xf5da42],
                  "Suguri": ["Suguri_(character)", 0x42D4F4]}


# noinspection PyTypeChecker
async def retrieve_image(character, color, short_name):
    async with ClientSession() as session:
        async with session.get(
                'https://gelbooru.com/index.php?page=dapi&s=post&q=index'
                '&tags=solo+-underwear+-sideboob+-pov_feet+-underboob+-upskirt+-sexually_suggestive+-ass+-bikini+'
                '-6%2Bgirls+-comic+-greyscale+-bdsm+-huge_filesize+-lovestruck+-absurdres+-artificial_vagina+'
                '-covering_breasts+-huge_breasts+-blood+-penetration_gesture+-animated+-audio+-webm+'
                'rating:safe+' + character) as request:
            if request.status == 200:
                await sleep(0.3)
                soup = BeautifulSoup(await request.text(), "lxml")
                num = int(soup.find('posts')['count'])
                max_pages = int(round(num / 100))
                page = randint(0, max_pages)
                all_posts = soup.find('posts')
                images = all_posts.find_all('post')
                # source = ((soup.find('post'))['source'])
                if num < 100:
                    image = images[randint(0, num - 1)]
                elif page == max_pages:
                    image = images[randint(0, 99)]
                else:
                    image = images[randint(0, 99)]
                url = image['file_url']
                # booru_id = image['id']
                # booru_tags = image['tags']
                booru_source = image['source']
                embed = Embed(title='', description=' ', colour=color)
                embed.set_author(name=short_name)
                embed.set_image(url=url)
                embed.set_footer(text="Source URL: " + booru_source)
                return embed
