""" Music related utils. """

from discord import FFmpegPCMAudio, PCMVolumeTransformer, Embed, Color
from asyncio import Event, Queue, get_event_loop, run_coroutine_threadsafe, QueueEmpty
from aya import nitori, config
from aya.utils import youtube, formatter

session_pool = {}


class PlayerSession:
    def __init__(self):
        self.client = None
        self.active = None
        self.audio = None
        self.queue_message = Embed(title="Queue", color=Color.dark_grey())
        self.processing = False
        self.play_next_song = Event()
        self.songs = Queue()
        self.skip_votes = []
        self.queue_message.add_field(name="Currently playing:", value="Nothing.")

    @property
    def player(self):
        return self.audio.player

    def playing(self):
        if self.client is None or self.audio is None:
            return False
        return not self.client.is_playing()

    async def cleanup(self):
        self.songs = Queue()
        if self.playing():
            try:
                self.client.stop()
                self.player.stop()
            except AttributeError:
                pass
            self.player.cleanup()
            self.audio.cleanup()
        self.audio = None
        await self.client.disconnect()


class YoutubeSource(PCMVolumeTransformer):

    def __init__(self, source, *, data, sound_volume=0.5):
        super().__init__(source, sound_volume)
        self.data = data
        self.title = data.get('title')
        self.url = data.get('url')

    @classmethod
    async def from_url(cls, url, *, loop=None, aria=False):
        loop = loop or get_event_loop()
        if aria:
            data = await loop.run_in_executor(None, youtube.youtube_dl_aria.extract_info, url)
        else:
            data = await loop.run_in_executor(None, youtube.youtube_dl_no_playlist.extract_info, url)
        if 'entries' in data:
            data = data['entries'][0]
        filename = (youtube.youtube_dl_no_playlist.prepare_filename(data))
        return cls(FFmpegPCMAudio(filename, before_options="-nostdin", options="-vn -reconnect 1"), data=data)


async def playlist(context, url: str):
    """ Adds a playlist to the queue. """
    gid = context.guild.id
    session = get_session(gid)
    information = youtube.extract_info(url, playlist=True)
    entry_amount = len(information["entries"])
    count = 0
    notification = await context.send(content=f"Adding {entry_amount} songs to playlist...")
    for entry in information["entries"]:
        count += 1
        try:
            await session.songs.put(entry["webpage_url"])
        except KeyError:
            await session.songs.put(f"https://www.youtube.com/watch?v={entry['url']}")
    if session.audio is not None:
        if session.client.is_playing():
            pass
    else:
        await get_next_song(context)
    await notification.edit(content=f"Added {entry_amount} songs to playlist.")


async def get_next_song(context):
    session = get_session(context.guild.id)
    if context.voice_client is None or session.audio is None:
        return
    session.audio.player.cleanup()
    session.audio.cleanup()
    try:
        url = session.songs.get_nowait()
    except QueueEmpty:
        return
    information = youtube.extract_info(url, playlist=False)
    if information is None:
        return await get_next_song(context)
    duration = information['duration']
    if duration > 3600 and not config.get("hour_video", False):
        await get_next_song(context)
        return await context.send("Videos longer than one hour is not allowed.")
    if config.get("aria", False):
        session.audio = await YoutubeSource.from_url(url, loop=nitori.loop, aria=True)
    else:
        session.audio = await YoutubeSource.from_url(url, loop=nitori.loop, aria=False)
    audio_source = PCMVolumeTransformer(session.audio)
    session.audio.player = audio_source
    session.queue_message = formatter.queue_formatter(session)
    await context.send(embed=formatter.music_formatter(information))
    context.voice_client.play(audio_source, after=lambda e: next_song(context))


def get_session(guild_id, create: bool = False):
    session = session_pool.get(guild_id, None)
    if session is None and create:
        session = PlayerSession()
        session_pool[guild_id] = session
    return session


def next_song(context):
    coroutine_object = get_next_song(context)
    run_coroutine_threadsafe(coroutine_object, nitori.loop)
